@@ -1,12 +1,13 @@
 import glob
 import pandas as pd
 from pylab import *
-from configparser import ConfigParser, MissingSectionHeaderError
+from configparser import ConfigParser, MissingSectionHeaderError, NoSectionError, NoOptionError
 import os
 import cv2
 import numpy as np
 from simba.drop_bp_cords import getBpHeaders
 import math
+from simba.rw_dfs import *
 
 def plotHeatMapLocation(inifile, animalbp1, mmSize, noIncrements, colorPalette, lastImageOnlyBol):
     config = ConfigParser()
@@ -26,17 +27,22 @@ def plotHeatMapLocation(inifile, animalbp1, mmSize, noIncrements, colorPalette,
     vidInfPath = os.path.join(projectPath, 'logs', 'video_info.csv')
     vidinfDf = pd.read_csv(vidInfPath)
     colorList, loopCounter = [], 0
+    try:
+        wfileType = config.get('General settings', 'workflow_file_type')
+    except NoOptionError:
+        wfileType = 'csv'
+    filesFound = glob.glob(csv_dir_in + '/*.' + wfileType)
+    print('Generating ' + str(len(filesFound)) + ' heatmaps...')
 
-    filesFound = glob.glob(csv_dir_in + '/*.csv')
     for currVid in filesFound:
         calcFlag = False
         loopCounter += 1
         currVidBaseName = os.path.basename(currVid)
-        currVidInfo = videoLog.loc[videoLog['Video'] == str(currVidBaseName.replace('.csv', ''))]
+        currVidInfo = videoLog.loc[videoLog['Video'] == str(currVidBaseName.replace('.' + wfileType, ''))]
         fps, width, height = int(currVidInfo['fps']), int(currVidInfo['Resolution_width']), int(currVidInfo['Resolution_height'])
         pxPerMM = float(currVidInfo['pixels/mm'])
         binInput = float(mmSize * pxPerMM)
-        outputfilename = os.path.join(frames_dir_out, currVidBaseName.replace('.csv', '.mp4'))
+        outputfilename = os.path.join(frames_dir_out, currVidBaseName.replace('.' + wfileType, '.mp4'))
         binWidth, binHeight = (binInput, binInput)
         try:
             NbinsX, NbinsY, NbinsXCheck, NbinsYCheck = (int(width / binWidth), int(height / binHeight), width / binWidth, height / binHeight)
@@ -45,12 +51,15 @@ def plotHeatMapLocation(inifile, animalbp1, mmSize, noIncrements, colorPalette,
         targetCountArrayFrames = np.zeros((NbinsY, NbinsX))
         im = np.zeros((height, width, 3))
         im.fill(0)
-        currDf = pd.read_csv(currVid)
+        currDf = read_df(currVid, wfileType)
+        try:
+            currDf = currDf.set_index('scorer')
+        except KeyError:
+            pass
         newColHeads = getBpHeaders(inifile)
-        newColHeads.insert(0, "scorer")
         currDf.columns = newColHeads
         count_row = currDf.shape[0]
-        vidInfo = vidinfDf.loc[vidinfDf['Video'] == str(currVidBaseName.replace('.csv', ''))]
+        vidInfo = vidinfDf.loc[vidinfDf['Video'] == str(currVidBaseName.replace('.' + wfileType, ''))]
         fps = int(vidInfo['fps'])
         im_scale_bar_width = int(width / 5)
         im_scale_bar = np.zeros((height, im_scale_bar_width, 3))
@@ -140,10 +149,6 @@ def plotHeatMapLocation(inifile, animalbp1, mmSize, noIncrements, colorPalette,
 
         for i in range(len(increments_digits_jump_list)):
             currY = im_scale_bar_digits_jumps_List[i]
-            # if i == 0:
-            #     currY = currY + 15
-            # if i == len(increments_digits_jump_list)-1:
-            #     currY = currY - 15
             if (i != 0) and (i != len(increments_digits_jump_list)-1):
                 currText = increments_digits_jump_list[i]
                 cv2.putText(im_scale_bar_digits, str(currText), (10, currY), cv2.FONT_HERSHEY_COMPLEX, fontScale, (255, 255, 255), 1)
@@ -182,9 +187,9 @@ def plotHeatMapLocation(inifile, animalbp1, mmSize, noIncrements, colorPalette,
             imageConcat = np.concatenate((im, im_scale_bar_text), axis=1)
             imageConcat = cv2.resize(imageConcat, (outputWidth, height))
             imageConcat = np.uint8(imageConcat)
-            imageSaveName = os.path.join(frames_dir_out, currVidBaseName.replace('.csv', '.png'))
+            imageSaveName = os.path.join(frames_dir_out, currVidBaseName.replace('.' + wfileType, '.png'))
             cv2.imwrite(imageSaveName, imageConcat)
-            print('Heatmap saved @: project_folder/frames/output/' + str(currVidBaseName.replace('.csv', '.png')))
+            print('Heatmap saved @: project_folder/frames/output/heatmap_location/' + str(currVidBaseName.replace('.' + wfileType, '.png')))
 
         if lastImageOnlyBol == 0:
             fourcc = cv2.VideoWriter_fourcc(*'mp4v')
